/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Contacts Sidebar.
 *
 * The Initial Developer of the Original Code is Jeroen Peters.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Jeroen Peters <jpeters@coldmail.nl>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gCollapsedContactsViewMode = true;
var msgComposeFormat = Components.interfaces.nsIMsgCompFormat;
var msgComposeType = Components.interfaces.nsIMsgCompType;

var gAddressBookBundle;
var gContactsPanelBundle;
var gContactsTree;
var gUseSecondEmail;
var gAccountManager;
var gIOService;
var gUseDefaultSearch;
var gDefaultSearch;
var gSearchInput;
var gUseMultipleComposers;
var gConfirmDelete;

// List/card selections in the results pane.
const kMultipleCardsOnly = 5;
const kSingleCardOnly = 6;

function InitializeGlobalVariables()
{
  gAccountManager = Components.classes['@mozilla.org/messenger/account-manager;1'].getService(Components.interfaces.nsIMsgAccountManager);
  gIOService = Components.classes['@mozilla.org/network/io-service;1'].getService(Components.interfaces.nsIIOService);
  
  gAddressBookBundle = document.getElementById("bundle_addressBook");
  gContactsPanelBundle = document.getElementById("bundle_contactsPanel");
  gContactsTree = document.getElementById("abResultsTree");
  gSearchInput = document.getElementById("searchInput");
  gUseSecondEmail = false;
  AddPrefObservers();
  
  gUseDefaultSearch = nsPreferences.getBoolPref("contactssidebar.auto_populate", true);
  gDefaultSearch = nsPreferences.copyUnicharPref("contactssidebar.auto_search_query", "*");
  gUseMultipleComposers = nsPreferences.getBoolPref("contactssidebar.multiple_composers", false);
  gConfirmDelete = nsPreferences.getIntPref("contactssidebar.confirm_delete", 1);
}


function ReleaseGlobalVariables()
{
  gAccountManager = null;
  gIOService = null;
}


function AddPrefObservers()
{
  var prefService = Components.classes['@mozilla.org/preferences-service;1']
                              .getService(Components.interfaces.nsIPrefService);
  var prefBranch = prefService.getBranch(null).QueryInterface(Components.interfaces.nsIPrefBranchInternal);

  prefBranch.addObserver("contactssidebar.search_mode",       ContactsSidebarPrefObserver, false);
  prefBranch.addObserver("contactssidebar.auto_populate",     ContactsSidebarPrefObserver, false);
  prefBranch.addObserver("contactssidebar.auto_search_query", ContactsSidebarPrefObserver, false);
  prefBranch.addObserver("contactssidebar.show_cc_item",      ContactsSidebarPrefObserver, false);
  prefBranch.addObserver("contactssidebar.show_bcc_item",     ContactsSidebarPrefObserver, false);
  prefBranch.addObserver("contactssidebar.multiple_composers",ContactsSidebarPrefObserver, false);
  prefBranch.addObserver("contactssidebar.confirm_delete",    ContactsSidebarPrefObserver, false);
}


function RemovePrefObservers()
{
  var prefService = Components.classes['@mozilla.org/preferences-service;1']
                              .getService(Components.interfaces.nsIPrefService);
  var prefBranch = prefService.getBranch(null).QueryInterface(Components.interfaces.nsIPrefBranchInternal);

  prefBranch.removeObserver("contactssidebar.search_mode",       ContactsSidebarPrefObserver);
  prefBranch.removeObserver("contactssidebar.auto_populate",     ContactsSidebarPrefObserver);
  prefBranch.removeObserver("contactssidebar.auto_search_query", ContactsSidebarPrefObserver);
  prefBranch.removeObserver("contactssidebar.show_cc_item",      ContactsSidebarPrefObserver);
  prefBranch.removeObserver("contactssidebar.show_bcc_item",     ContactsSidebarPrefObserver);
  prefBranch.removeObserver("contactssidebar.multiple_composers",ContactsSidebarPrefObserver);
  prefBranch.removeObserver("contactssidebar.confirm_delete",    ContactsSidebarPrefObserver);
}


function ToggleContactsView() {
  var expandedNode = document.getElementById("expandedContactsView");
  var collapsedNode = document.getElementById("collapsedContactsView");
  var toggleContactsView = document.getElementById("contactsView");

  if (gCollapsedContactsViewMode) {          
    gCollapsedContactsViewMode = false;

    // now uncollapse / collapse the right views
    expandedNode.collapsed = false;
    collapsedNode.collapsed = true;
  } else {
    gCollapsedContactsViewMode = true;

    // now uncollapse / collapse the right views
    collapsedNode.collapsed = false;
    expandedNode.collapsed = true;
  }  

  if (gCollapsedContactsViewMode)
    toggleContactsView.setAttribute("state", "true");
  else
    toggleContactsView.setAttribute("state", "false");
}


const ContactsSidebarPrefObserver = {
  observe: function(subject, topic, prefName) {
    // verify that we're changing the contacts sidebar config pref
    if (topic == "nsPref:changed") {
      switch (prefName)
      {
        case "contactssidebar.auto_populate":
          gUseDefaultSearch = nsPreferences.getBoolPref(prefName, true);
          break;

        case "contactssidebar.auto_search_query":
          gDefaultSearch = nsPreferences.copyUnicharPref(prefName, "*");
          break;

        case "contactssidebar.show_cc_item":
        case "contactssidebar.show_bcc_item":
          setMenuOptionsVisibility();
          break;
      
        case "contactssidebar.multiple_composers":
          gUseMultipleComposers = nsPreferences.getBoolPref(prefName, true);
          break;
        case "contactssidebar.confirm_delete":
          gConfirmDelete = nsPreferences.getIntPref(prefName, 1);
          break;
      }
    }
  }
};


function contactsPanelLoad() {
  InitializeGlobalVariables();
  AbPanelLoad();
  document.loadBindingDocument('chrome://global/content/bindings/textbox.xml');
  
  setMenuOptionsVisibility();
  
  document.getElementById("cardProperties").addEventListener("popupshowing", setContext, false);
  
  var collapsedNode = document.getElementById("collapsedContactsView");
  if (collapsedNode.collapsed)  
    ToggleContactsView();
}


function contactsPanelUnload() {
  RemovePrefObservers();
  ReleaseGlobalVariables();
  AbPanelUnload();
}


function setContext() {
  var writeItem = document.getElementById("writeItem");
  var writeMenu = document.getElementById("writeMenu");
  
  var selectedCards = GetSelectedAbCards();
  //var hasAddEmail = selectedCards.length == 1 && selectedCards[0].secondEmail != "";
  var hasAddEmail = cardsHasSecondEmail(selectedCards);
  
  setWriteSubMenus("writeItem", "writeMenu", "primaryEmailItem", "additionalEmailItem",
    hasAddEmail, selectedCards);
    
  if ( nsPreferences.getBoolPref("contactssidebar.show_cc_item",  false) )
    setWriteSubMenus("writeCcItem", "writeCcMenu", "primaryEmailCcItem", "additionalEmailCcItem",
      hasAddEmail, selectedCards);
  
  if ( nsPreferences.getBoolPref("contactssidebar.show_bcc_item",  false) )
  setWriteSubMenus("writeBccItem", "writeBccMenu", "primaryEmailBccItem", "additionalEmailBccItem",
    hasAddEmail, selectedCards);

  // Disable the "properties" item if multiple contacts are selected
  if (selectedCards.length == 1)
    document.getElementById("propertiesItem").removeAttribute("disabled");
  else
    document.getElementById("propertiesItem").setAttribute("disabled", true);    
}


function cardsHasSecondEmail(cards) {
  var result = false;
  
  for (var i=0; i<cards.length; i++)
  {
    result = cards[i].secondEmail != "";
    if (result)
      break;  
  }
  
  return result;
}


function setWriteSubMenus(item, menu, primary, additional, hasAddEmail, cards) {
  if (hasAddEmail)
  {
    document.getElementById(item).hidden = true;
    document.getElementById(menu).hidden = false;
    
    if (cards.length == 1)
    {
      document.getElementById(primary).label = cards[0].primaryEmail;
      document.getElementById(additional).label = cards[0].secondEmail;
    }
    else 
    {
      document.getElementById(primary).label = gContactsPanelBundle.getString("contactsSidebar.email");
      document.getElementById(additional).label = gContactsPanelBundle.getString("contactsSidebar.additionalEmail");
    }
  }
  else
  {
    document.getElementById(item).hidden = false;
    document.getElementById(menu).hidden = true;
  }
}


function setMenuOptionsVisibility() {
  var ccValue  = nsPreferences.getBoolPref("contactssidebar.show_cc_item",  false);
  var bccValue = nsPreferences.getBoolPref("contactssidebar.show_bcc_item", false);
  
  setMenuOptionVisibility("writeCcItem", "writeCcMenu",  ccValue);
  setMenuOptionVisibility("writeBccItem", "writeBccMenu", bccValue);
}


function setMenuOptionVisibility(menuId, submenuId, prefValue) {
  var elem = document.getElementById(menuId);
  if (elem) {
    elem.hidden = !prefValue;
  }
  if (submenuId != "")
    var elem = document.getElementById(submenuId);
    if (elem) {
      elem.hidden = !prefValue;
    }

  return prefValue;
}


function isValidRow(aEvent, doSelect)
{
  var row = gContactsTree.treeBoxObject.getRowAt(aEvent.clientX, aEvent.clientY);
  
  var result = (row != -1 &&
                row <= gContactsTree.view.rowCount-1 &&
                aEvent.originalTarget.localName == "treechildren")
  
  if (result && doSelect)
  {
    gContactsTree.focus();
    gContactsTree.view.selection.select(row);
  }
    
  return result;
}


function contactsPanelOnClick(event)
{
  // we only care about button 0 (left click) and 
  // button 1 (middle click) events
  if (event.button == 0) 
  {
    // all we need to worry about here is column header clicks.
    var t = event.originalTarget;

    if (t.localName == "treecol") {
      var sortDirection;
      var currentDirection = t.getAttribute("sortDirection");

      if (currentDirection == kDefaultDescending)
        sortDirection = kDefaultAscending;
      else
        sortDirection = kDefaultDescending;

      SortAndUpdateIndicators(t.id, sortDirection);
    }
  }
  else if (event.button == 1) 
  {
    processClickEvent(event);
  }
}


function contactsPanelOnDoubleClick(event)
{
  // we only care about button 0 (left click) events
  if (event.button == 0)
    processClickEvent(event);
}


function processClickEvent(event)
{
  // Only select valid row if middle button is clicked
  if ( !isValidRow(event, event.button == 1) )
  {
    // clicking on a non valid row should not open the compose window
    return;
  }

  // Alt double click: view properties
  if (event.altKey)
  {
    AbEditSelectedCard();
    return;
  }
  
  // Shift double click toggles HTML/text compose
  if (event.shiftKey)
    var format = msgComposeFormat.OppositeOfDefault;
  else
    var format = msgComposeFormat.Default;
    
  // Control double click selects second email address as to: address
  var useSecondEmail = event.ctrlKey

  // ok, go ahead and write a new message
  contactsComposeMessage(msgComposeType.New, format, 'addr_to', useSecondEmail)
}


function onContactsKey(aEvent)
{
  switch( aEvent.keyCode ) 
  {
    case KeyEvent.DOM_VK_ENTER:
    case KeyEvent.DOM_VK_RETURN:
      if (aEvent.altKey)
      {
        AbEditSelectedCard();
      }
      else {
        // Ctrl+enter uses additional email address
        var useSecondEmail = aEvent.ctrlKey;
      
        // Shift+enter toggles HTML/text compose
        if (aEvent.shiftKey)
          var format = msgComposeFormat.OppositeOfDefault;
        else
          var format = msgComposeFormat.Default;

        contactsComposeMessage(msgComposeType.New, format, 'addr_to', useSecondEmail) 
      }
      break;
    case KeyEvent.DOM_VK_DELETE:
      AbDelete();
      aEvent.preventDefault();
      break;
    case KeyEvent.DOM_VK_INSERT:
      AbNewCard('dirTree');
      break;
  }
}


function contactsComposeMessage(type, format, recipientType, useSecondEmail, attachments, subject)
{
  var msgComposeType = Components.interfaces.nsIMsgCompType;
  var msgComposFormat = Components.interfaces.nsIMsgCompFormat;
  var msgComposeService = Components.classes['@mozilla.org/messengercompose;1'].getService();
  msgComposeService = msgComposeService.QueryInterface(Components.interfaces.nsIMsgComposeService);

  var params = Components.classes['@mozilla.org/messengercompose/composeparams;1'].createInstance(Components.interfaces.nsIMsgComposeParams);
  if (params)
  {
    params.type = type;
    params.format = msgComposFormat.Default;
    var composeFields = Components.classes['@mozilla.org/messengercompose/composefields;1'].createInstance(Components.interfaces.nsIMsgCompFields);
    if (composeFields) 
    {
      gUseSecondEmail = useSecondEmail;
      var addresses = GetSelectedAddresses();
      switch (recipientType) {
        case 'addr_to': 
          composeFields.to = addresses;
          break;
        case 'addr_cc': 
          composeFields.cc = addresses;
          break;
        case 'addr_bcc': 
          composeFields.bcc = addresses;
          break;
      }
      // Add attachment
      if (attachments)
      {
        if (attachments[0])
          for (var i=0; i<attachments.length; i++)
            composeFields.addAttachment(attachments[i]);
        else
          composeFields.addAttachment(attachments);
      }
      
      if (subject)
        composeFields.subject = subject;
        
      var identity;
      var selectedServer = window.parent.GetSelectedServer();
      if (selectedServer)
        identity = gAccountManager.getFirstIdentityForServer(selectedServer);
      if (!identity)
        identity = gAccountManager.defaultAccount.defaultIdentity;
      if (identity)
        params.identity = identity;
      
      params.composeFields = composeFields;
      params.format = format;
      
      msgComposeService.OpenComposeWindowWithParams(null, params);
    }
  }
}


function GetSelectedCardTypes()
{
  var cards = GetSelectedAbCards();
  if (!cards)
    return kNothingSelected; // no view

  var count = cards.length;
  if (!count)
    return kNothingSelected;  // nothing selected

  var mailingListCnt = 0;
  var cardCnt = 0;
  for (var i = 0; i < count; i++) {
    if (cards[i].isMailList)
      mailingListCnt++;
    else
      cardCnt++;
  }
  if (mailingListCnt && cardCnt)
    return kListsAndCards;        // lists and cards selected
  else if (mailingListCnt && !cardCnt) {
    if (mailingListCnt > 1)
      return kMultipleListsOnly; // only multiple mailing lists selected
    else
      return kSingleListOnly;    // only single mailing list
  }
  else if (!mailingListCnt && cardCnt)
    if (cardCnt > 1)
      return kMultipleCardsOnly; // only multiple cards selected
    else
      return kSingleCardOnly;    // only single card list
}


function AbDelete()
{
  var types = GetSelectedCardTypes();
  if (types == kNothingSelected)
    return;

  var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);

  // Only ask for confirmation if this is wanted
  if (gConfirmDelete != 2)
  {
    var confirmDeleteMessage = "";
    
    // If at least one mailing list is selected then prompt users for deletion.
    if (types == kListsAndCards)
      confirmDeleteMessage = gContactsPanelBundle.getString("contactsSidebar.confirmDeleteListsAndCards");
    else if (types == kMultipleListsOnly)
      confirmDeleteMessage = gContactsPanelBundle.getString("contactsSidebar.confirmDeleteMailingLists");
    else if (types == kSingleListOnly)
      confirmDeleteMessage = gContactsPanelBundle.getString("contactsSidebar.confirmDeleteMailingList");
      
    // If confirm delete is wanted for all contacts then prompt users for deletion.
    else if (gConfirmDelete == 0 && types == kMultipleCardsOnly)
      confirmDeleteMessage = gContactsPanelBundle.getString("contactsSidebar.confirmDeleteCards");
    else if (gConfirmDelete == 0 && types == kSingleCardOnly)
      confirmDeleteMessage = gContactsPanelBundle.getString("contactsSidebar.confirmDeleteCard");

    if (confirmDeleteMessage != "")
      if (!promptService.confirm(window, null, confirmDeleteMessage))
        return;
  }

  gAbView.deleteSelectedCards();
}


function GenerateAddressFromCard(card)
{
  if (!card)
    return "";

  var email = "";

  if (card.isMailList)
  {
    var directory = GetDirectoryFromURI(card.mailListURI);
    if(directory.description)
      email = directory.description;
    else
      email = card.displayName;
  }
  else {
    if (gUseSecondEmail)
      email = card.secondEmail;
      
    if (email == "")
      email = card.primaryEmail;
  }

  return gHeaderParser.makeFullAddressWString(card.displayName, email);
}


var contactsTreeObserver = {

  canHandleMultipleItems: true,

  onDrop: function (aEvent, aData, aDragSession)
    {
      var attachments = [];
      var k = 0;

      contactsPanelOnClick(aEvent);

      var dataList = aData.dataList;
      var dataListLength = dataList.length;
      var attachment;
      
      for (var i = 0; i < dataListLength; i++) 
      {
        var item = dataList[i].first;
        var prettyName;
        var rawData = item.data;
        var type = msgComposeType.New;
        
        if (item.flavour.contentType == "text/x-moz-url" ||
            item.flavour.contentType == "text/x-moz-message" ||
            item.flavour.contentType == "application/x-moz-file")
        {
          if (item.flavour.contentType == "application/x-moz-file")
          {
            var ioService = Components.classes['@mozilla.org/network/io-service;1']
                            .getService(Components.interfaces.nsIIOService);
            var fileHandler = ioService.getProtocolHandler("file")
                              .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
            rawData = fileHandler.getURLSpecFromFile(rawData);
          }
          else if (item.flavour.contentType == "text/x-moz-url")
          {
            var separator = rawData.indexOf("\n");
            if (separator != -1) 
            {
              prettyName = rawData.substr(separator+1);
              rawData = rawData.substr(0,separator);
            }
          }

          var isValid = true;
          if (item.flavour.contentType == "text/x-moz-url" ||
              item.flavour.contentType == "text/x-moz-message") {
            // if this is a url (or selected text) see if it's a valid url by checking 
            // if we can extract a scheme using the ioservice
            //
            // also skip mailto: since it doesn't make sense to attach and send mailto urls
            try {
              var scheme = gIOService.extractScheme(rawData);
              // don't attach mailto: urls
              if (scheme == "mailto")
                isValid = false;
            }
            catch (ex) {
              isValid = false;
            }
            if (isValid) {
              attachment = Components.classes['@mozilla.org/messengercompose/attachment;1']
                                    .createInstance(Components.interfaces.nsIMsgAttachment);
              attachment.url = rawData;
              attachment.name = prettyName;
              attachments[k++] = attachment;
              
              if (gUseMultipleComposers)
              {
                if (item.flavour.contentType == "text/x-moz-message")
                {
                  csMsgForwardMessage(attachment);
                }
                else
                  contactsComposeMessage(type, msgComposeFormat.Default, 'addr_to', false, attachment);
              }
            }
          }
        }
      }
    
      if (!gUseMultipleComposers)
      {
        if (item.flavour.contentType == "text/x-moz-message")
          csMsgForwardMessage(attachments);
        else        
          contactsComposeMessage(type, msgComposeFormat.Default, 'addr_to', false, attachments);
      }
    },

  onDragOver: function (aEvent, aFlavour, aDragSession)
    {
      gContactsTree.setAttribute("dragover", "true");
      
      // Select row (if valid)
      isValidRow(aEvent, true);
    },

  onDragExit: function (aEvent, aDragSession)
    {
      gContactsTree.removeAttribute("dragover");
    },

  getSupportedFlavours: function ()
    {
      var flavourSet = new FlavourSet();
      flavourSet.appendFlavour("text/x-moz-url");
      flavourSet.appendFlavour("text/x-moz-message");
      flavourSet.appendFlavour("application/x-moz-file", "nsIFile");
      return flavourSet;
    }
};


function DirPaneSelectionChangeMenulist()
{
  if (abList && abList.selectedItem) 
  {
    if (gSearchInput.value && (gSearchInput.value != ""))
      onEnterInSearchBar();
    else
      ChangeDirectoryByURI(abList.selectedItem.id);

    if ( isLdapDirectory(abList.selectedItem.id) )
      // Disable delete item for LDAP address books
      document.getElementById("deleteItem").setAttribute("disabled", true);
    else
      // Enable delete item for "normal" address books
      document.getElementById("deleteItem").removeAttribute("disabled");
  }
}


function onCsSearchInputFocus(event)
{
  // search bar has focus, ...clear the showing search criteria flag
  if (gSearchInput.showingSearchCriteria)
  {
    gSearchInput.value = "";
    gSearchInput.showingSearchCriteria = false;
  }

  gSearchInput.select();
}


function onCsSearchInputBlur(event)
{
  // ignore the blur if we are in the middle of processing the clear button
  if (gSearchFocusEl && gSearchFocusEl.id == 'searchInput')
    return;
               
  if (!gSearchInput.value)
    gSearchInput.showingSearchCriteria = true;

  if (gSearchInput.showingSearchCriteria)
    gSearchInput.setSearchCriteriaText();
}


function onCsSearchInput(returnKeyHit)
{
  if (gSearchInput.showingSearchCriteria)
    return;

  if (gSearchTimer) {
    clearTimeout(gSearchTimer);
    gSearchTimer = null;
  }

  if (returnKeyHit) {
    onEnterInSearchBar();
  }
  else {
    gSearchTimer = setTimeout("onEnterInSearchBar();", 800);
  }
}


// temporary global used to make sure we restore focus to the correct element after 
// clearing the quick search box because clearing quick search means stealing focus.
var gSearchFocusEl = null;

function onCsClearSearch()
{
  // ignore the text box value if it's just showing the search criteria string
  if (!gSearchInput.showingSearchCriteria) 
  {
    // save of the last focused element so that focus can be restored
    //gSearchFocusEl = gLastFocusedElement;
    
    gSearchInput.value = "";
    onEnterInSearchBar();
    
    // this needs to be on a timer otherwise we end up messing up the focus while 
    // the Search("") is still happening
    setTimeout("restoreSearchFocusAfterClear();", 0);
  }
}


function restoreSearchFocusAfterClear()
{
//  gSearchFocusEl.focus();
  gSearchInput.clearButtonHidden = 'true';
  gSearchFocusEl = null;
}


function onCsSearchKeyPress(event)
{
  if (gSearchInput.showingSearchCriteria)
    gSearchInput.showingSearchCriteria = false;

  if (event && event.keyCode == KeyEvent.DOM_VK_RETURN)
    onCsSearchInput(true);
  // added this else clause, because the oninput event is never called. It looks
  // this is because in the XUL file the parent vbox is initial collapsed. 
  else
    onCsSearchInput(false);
}


function isValidSearchQuery()
{
  return !(gSearchInput.value == "" || gSearchInput.showingSearchCriteria);
}


function onEnterInSearchBar()
{
  switch (gSearchInput.searchMode)
  {
    case "0":
      // Name or Company
      gQueryURIFormat = "?(or(DisplayName,c,@V)(FirstName,c,@V)(LastName,c,@V)(Company,c,@V))";
      break;
    case "1":
      // Email addresses
      gQueryURIFormat = "?(or(PrimaryEmail,c,@V)(SecondEmail,c,@V))";
      break;
    case "2":
      // Phone numbers
      gQueryURIFormat = "?(or(WorhPhone,c,@V)(HomePhone,c,@V)(CellularNumber,c,@V))";
      break;
    case "3":
      // All Fields
      gQueryURIFormat = "?(or(DisplayName,c,@V)(FirstName,c,@V)(LastName,c,@V)(Company,c,@V)(PrimaryEmail,c,@V)(SecondEmail,c,@V)(WorhPhone,c,@V)(HomePhone,c,@V)(CellularNumber,c,@V))";
      break;
   }

  var searchURI = GetSelectedDirectory();
  if (!searchURI) return;
  
  var sortColumn    = gAbView ? gAbView.sortColumn    : kDefaultSortColumn;
  var sortDirection = gAbView ? gAbView.sortDirection : kDefaultAscending;
  var searchTerm = gSearchInput.value;

  // When switching TO an LDAP directory, use a default 
  // search term so no empty list appears
  if ( gUseDefaultSearch && isLdapDirectory(searchURI) && !isValidSearchQuery() )
    searchTerm = gDefaultSearch;
  
  if (isValidSearchQuery() || searchTerm == gDefaultSearch)
    searchURI += gQueryURIFormat.replace(/@V/g, encodeURIComponent(searchTerm));

  SetAbView(searchURI, searchTerm != "", sortColumn, sortDirection);
  
  if (!isValidSearchQuery(gSearchInput.value))
    gSearchInput.showingSearchCriteria = true;
  else
    gSearchInput.clearButtonHidden = 'false';
  
  SelectFirstCard();
}


function isLdapDirectory(uri)
{
  var ldapUrlPrefix = "moz-abldapdirectory://";
  result = ((uri.indexOf(ldapUrlPrefix, 0)) == 0)
  
  return result;
}

function ChangeDirectoryByURI(uri)
{
  if (!uri)
    uri = kPersonalAddressbookURI;

  if (gAbView && gAbView.URI == uri)
    return;

  var dataNode = document.getElementById(uri);
  var sortColumn = dataNode ? dataNode.getAttribute("sortColumn") : kDefaultSortColumn;
  var sortDirection = dataNode ? dataNode.getAttribute("sortDirection") : kDefaultAscending;

  //Switching to LDAP -> use default search term
  if ( isLdapDirectory(dataNode.value) )
  {
    onEnterInSearchBar();
    return;
  }
  
  var actualSortColumn = SetAbView(uri, sortColumn, sortDirection);

  UpdateSortIndicators(actualSortColumn, sortDirection);

  // only select the first card if there is a first card
  if (gAbView && gAbView.getCardFromRow(0)) {
    SelectFirstCard();
  }
  else {
    // the selection changes if we were switching directories.
    ResultsPaneSelectionChanged()
  }
  return;
}

function changeCsSearchMode(aMenuItem)
{
  // extract the label and set the search input to match it
  var oldSearchMode = gSearchInput.searchMode;
  gSearchInput.searchMode = aMenuItem.value;

  if ( !isValidSearchQuery() )
  {
    gSearchInput.showingSearchCriteria = true;
    if (gSearchInput.value)
      gSearchInput.setCsSearchCriteriaText();
  }
  
  if (!gSearchInput.value)
    // if the search box is empty, set showing search criteria to true so it 
    // shows up when focus moves out of the box
    gSearchInput.showingSearchCriteria = true;
  else if (gSearchInput.showingSearchCriteria)
    // if we are showing criteria text and the box isn't empty, change the criteria text  
    gSearchInput.setSearchCriteriaText();
  else if (oldSearchMode != gSearchInput.searchMode)
    // the search mode just changed so we need to redo the quick search
    onEnterInSearchBar();
}

function cardToFilename(card, ext) {
  var result = "";
  
  if (card.primaryEmail != "")
    result = card.primaryEmail;
  else if (card.secondEmail != "")
    result = card.secondEmail;
  else
    result = "nsmail";
    
  var index = result.indexOf("@");
  if (index >= 0)
    result = result.substr(0, index);
  else
    result = result;
    
  result += "." + ext;
  
  return result;
}

function forwardContact() {
  var cards = GetSelectedAbCards();
  
  if (cards.length == 0)
    return;

  var subject = "";
  var prettyName = "";
  if (cards.length == 1)
  {
    subject = gContactsPanelBundle.getFormattedString("contactsSidebar.forwardContactSubject", [cards[0].displayName]);
    prettyName = cardToFilename(cards[0], "vcf");
  }
  else
  {
    subject = gContactsPanelBundle.getString("contactsSidebar.forwardMultipleContactSubject");
    prettyName = "contacts.vcf";
  }
  
  // Create vCard attachment 
  var vCardUrl = "data:text/x-vcard;charset=utf-8,";
  for (var i=0; i<cards.length; i++)
  {
    vCardUrl += encodeURIComponent(unescape( cards[i].convertToEscapedVCard() ));
  }
  
  // Create a message with vCard(s) as attachment
  attachment = Components.classes['@mozilla.org/messengercompose/attachment;1']
                        .createInstance(Components.interfaces.nsIMsgAttachment);
                        
  attachment.url = vCardUrl;
  attachment.name = prettyName;
  var attachments = new Array(attachment);
  
  contactsComposeMessage(msgComposeType.New, msgComposeFormat.Default, 'none', false, attachments, subject);
}


function copyContact(destABook)
{
  var destURI = destABook.getAttribute('id');
  var directory = GetDirectoryFromURI(destURI);

  var cards = GetSelectedAbCards();
  
  if (!cards)
    return;
    
  var count = cards.length;
  for (var i = 0; i < count; i++) 
  {
    if (cards[i].isMailList)
    {
      var mailList = GetDirectoryFromURI(cards[i].mailListURI);
      directory.addMailList(mailList);
/*
 test: copy the maillist members "by hand"
 
      var mailList = GetDirectoryFromURI(cards[i].mailListURI);
      while (mailList.childNodes.hasMoreElements() )
      {
        var currentCard = mailList.childNodes.getNext().QueryInterface(Components.interfaces.nsIAbCard);
        directory.dropCard(currentCard, true);
      }
*/      
    }
    else
      directory.dropCard(cards[i], true);
  }
  
}
